@@ -1,21 +1,29 @@
 import argparse
+import logging
+import os
 import sys
+import time
 
 from esmecata.proteomes import retrieve_proteomes
 from esmecata.clustering import make_clustering
 from esmecata.annotation import annotate_proteins
 from esmecata.workflow import perform_workflow
-from esmecata.utils import limited_integer_type, range_limited_float_type
+from esmecata.utils import limited_integer_type, range_limited_float_type, is_valid_dir
 from esmecata import __version__ as VERSION
 
 MESSAGE = '''
-From taxonomic annotation to metabolism using Uniprot.
+From taxonomic affiliation to metabolism using Uniprot.
 '''
 REQUIRES = '''
 Requires: mmseqs2 and an internet connection (for REST and SPARQL queries, except if you have a local Uniprot SPARQL endpoint).
 '''
 
+logger = logging.getLogger()
+logger.setLevel(logging.DEBUG)
+
 def main():
+    start_time = time.time()
+
     parser = argparse.ArgumentParser(
         'esmecata',
         description=MESSAGE + ' For specific help on each subcommand use: esmecata {cmd} --help',
@@ -32,7 +40,7 @@ def main():
         '--input',
         dest='input',
         required=True,
-        help='Input taxon file (excel, tsv or csv) containing a column associating ID to a taxonomic annotation (separated by ;).',
+        help='Input taxon file (excel, tsv or csv) containing a column associating ID to a taxonomic affiliation (separated by ;).',
         metavar='INPUT_FILE')
 
     parent_parser_i_clustering_folder = argparse.ArgumentParser(add_help=False)
@@ -84,7 +92,7 @@ def main():
     parent_parser_all_proteomes.add_argument(
         '--all-proteomes',
         dest='all_proteomes',
-        help='Download all proteomes associated to a taxon even if they are no reference proteomes.',
+        help='Download all proteomes associated with a taxon even if they are no reference proteomes.',
         required=False,
         action='store_true',
         default=None)
@@ -128,7 +136,7 @@ def main():
     parent_parser_linclust.add_argument(
         '--linclust',
         dest='linclust',
-        help='Use mmseqs linclust (clustering in lienar time) to cluster proteins sequences. It is faster than mmseqs cluster (default behaviour) but less senstitive.',
+        help='Use mmseqs linclust (clustering in lienar time) to cluster proteins sequences. It is faster than mmseqs cluster (default behaviour) but less sensitive.',
         required=False,
         action='store_true',
         default=None)
@@ -145,7 +153,7 @@ def main():
         '-p',
         '--propagate',
         dest='propagate_annotation',
-        help='Proportion [0 to 1] of the reccurence of an annotation to be propagated from the protein of a cluster to the reference protein of the cluster. 0 mean the annotaitons from all proteins are propagated to the reference and 1 only the annotation occurring in all the proteins of the cluster.',
+        help='Proportion [0 to 1] of the occurrence of an annotation to be propagated from the protein of a cluster to the reference protein of the cluster. 0 mean the annotations from all proteins are propagated to the reference and 1 only the annotation occurring in all the proteins of the cluster (default).',
         required=False,
         type=range_limited_float_type,
         default=1)
@@ -153,7 +161,7 @@ def main():
     parent_parser_uniref.add_argument(
         '--uniref',
         dest='uniref',
-        help='Use uniref cluster to extract more annotations from the representative member of the cluster associated to the proteins. Needs the --sparql option.',
+        help='Use uniref cluster to extract more annotations from the representative member of the cluster associated with the proteins. Needs the --sparql option.',
         required=False,
         action='store_true',
         default=None)
@@ -161,7 +169,7 @@ def main():
     parent_parser_expression.add_argument(
         '--expression',
         dest='expression',
-        help='Extract expresion information associated to the proteins. Needs the --sparql option.',
+        help='Extract expression information associated with the proteins. Needs the --sparql option.',
         required=False,
         action='store_true',
         default=None)
@@ -197,7 +205,7 @@ def main():
 
     proteomes_parser = subparsers.add_parser(
         'proteomes',
-        help='Download proteomes associated to taxon from Uniprot Proteomes.',
+        help='Download proteomes associated with taxon from Uniprot Proteomes.',
         parents=[
             parent_parser_i_taxon, parent_parser_o, parent_parser_b,
             parent_parser_taxadb, parent_parser_all_proteomes, parent_parser_sparql,
@@ -238,6 +246,21 @@ def main():
 
     args = parser.parse_args()
 
+    is_valid_dir(args.output)
+
+    # add logger in file
+    formatter = logging.Formatter('%(message)s')
+    log_file_path = os.path.join(args.output, f'esmecata_{args.cmd}.log')
+    file_handler = logging.FileHandler(log_file_path, 'w+')
+    file_handler.setLevel(logging.INFO)
+    file_handler.setFormatter(formatter)
+    logger.addHandler(file_handler)
+    # set up the default console logger
+    console_handler = logging.StreamHandler(sys.stdout)
+    console_handler.setLevel(logging.INFO)
+    console_handler.setFormatter(formatter)
+    logger.addHandler(console_handler)
+
     # If no argument print the help.
     if len(sys.argv) == 1:
         parser.print_help()
@@ -271,5 +294,9 @@ def main():
                             args.linclust, args.propagate_annotation, args.uniref,
                             args.expression, args.beta)
 
+    logger.info("--- Total runtime %.2f seconds ---" % (time.time() - start_time))
+    logger.warning(f'--- Logs written in {log_file_path} ---')
+
+
 if __name__ == '__main__':
     main()
